%% Code for the paper "Image reconstruction of compressed sensing MRI using graph-based redundant wavelet transform"
%% The L1 norm minimization problem is solved via an alternating direction method with continuation
%% Zongying Lai, Xiaobo Qu
%% Xiamen University
%% Email: quxiaobo <at> xmu.edu.cn
%% Nov.13, 2015
%% If you use this code, please cite our papers:
%% [1] Zongying Lai#, Xiaobo Qu#, Yunsong Liu, Di Guo, Jing Ye, Zhifang Zhan, Zhong Chen. Image reconstruction of compressed sensing MRI using graph-based redundant wavelet transform, Medical Image Analysis, 27: 93-104, 2015. (# denotes co-first authorship)
clc
clear all
close all
%%
path_GBRWT
%% %%%%%%%%%%%%%% load data for experiments
load T2wBrain_slice_10
kspace_full=fft2c(im_ori);
load mask_2D_Circle_015
SamplingRate=sum(mask(:))/numel(mask)
%% %%%%%%%%%%%%%%  Measurements
[row,column]=size(kspace_full);
y=mask_downsample(mask,row,column)*kspace_full;   
%% Shift invariant discrete wavelet transform (SIDWT)-based sparse MRI image reconstruction
paramsSIDWT.lamda=5*1e3;

filterType='Daubechies';
filterSize=4;
wavScale=4;
complex_Yes=1;
paramsSIDWT.psi=SIDWT(filterType,filterSize,wavScale,complex_Yes); % define SIDWT operator

tic
im_rec_SIDWT=solver_ADMC(y,mask,paramsSIDWT); % solve the L1 norm minimization problem with ADMC
time_SIDWT_CS=toc
Eval_SIDWT=Evaluation_CS_MRI(im_ori,im_rec_SIDWT)
imwrite(abs(im_rec_SIDWT),'im_rec_SIDWT.png');
%% Patch-based Directional Wavelets (PBDW)-based sparse MRI image reconstruction
paramsPBDW.lamda=5*1e3;
paramsPBDW.iter_optDirections=2;% Times to train the geometric directions
paramsPBDW.ref_init=abs(im_rec_SIDWT);% Incorporate the guide image.
paramsPBDW.ShowGeo=0;% Show the geometric directions? Yes:1, No:0
paramsGeo.overlapFactor=16;
paramsGeo.patchSize=8;
tic
im_rec_PBDW=solver_PBDW_CSMRI(y,mask,paramsPBDW,paramsGeo);
time_PBDW_CS=toc
Eval_PBDW=Evaluation_CS_MRI(im_ori,im_rec_PBDW)
imwrite(abs(im_rec_PBDW),'im_rec_PBDW.png');
%% Graph-based redundant wavelet transform (GBRWT)-based sparse MRI image reconstruction
paraGBRWT.lamda=5*1e3;
paraGBRWT.opt_max =2;% Times to train the graph
im_ref=abs(im_rec_SIDWT);% Incorporate the guide image.
tic
im_rec_GBRWT=Solve_RTBWT(y,mask,paraGBRWT,im_ref);
time_GBRWT_CS=toc
Eval_GBRWT=Evaluation_CS_MRI(im_ori,im_rec_GBRWT)
imwrite(abs(im_rec_GBRWT),'im_rec_GBRWT.png');
%%
figure,imshow(abs([im_ori-im_rec_SIDWT im_ori-im_rec_PBDW im_ori-im_rec_GBRWT ]), [0 0.1]), title('Error images')
%%
save project_GraphMRI_Recon
%%